{% extends 'base.html' %}
{% block title %}{{ page_title }}{% endblock %}
{% block main %}
  <div id="volumes-type">
      {{ volume_types_table.render }}
  </div>

  <div id="qos-specs">
      {{ qos_specs_table.render }}
  </div>
{% endblock %}