{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}

{% block modal-body-right %}
  <h3>{% trans "Description:" %}</h3>
  <p>{% blocktrans trimmed %}
    Create a new "spec" key-value pair for QoS Spec "{{ qos_spec_name }}".
    Valid key names are expected in the QoS specs. Refer to Cinder documentation
    for the supported front-end and back-end keys.
    {% endblocktrans %}
  </p>
  <p>
    {% trans "Front-End Example:" %}
    <ul>
      <li>{% trans "Key: total_iops_sec and Value: 5000" %}</li>
      <li>{% trans "Key: total_bytes_sec_max and Value: 512000" %}</li>
      <li>{% trans "Key: size_iops_sec and Value: 16" %}</li>
    </ul>
  </p>
  <p>
    {% trans "Back-End Example:" %}
    <ul>
      <li>{% trans "Key: minIOPS and Value: 20 (number value less than maxIOPS)" %}</li>
      <li>{% trans "Key: maxIOPS and Value: 5000 (number value bigger than minIOPS)" %}</li>
      <li>{% trans "Key: burstIOPS and Value: 5000 (number value bigger than minIOPS)" %}</li>
    </ul>
  </p>
{% endblock %}
