{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Create Protocol" %}{% endblock %}

{% block main %}
    {% include 'identity/identity_providers/protocols/_create.html' %}
{% endblock %}
