.transfer-table {
  .fa[title] {
    cursor: pointer;
  }

  .transfer-heading{
    @extend h4;
    font-size: $font-size-h4;

      .help-text{
        @extend h5;
        font-size: $font-size-h5;

      }
  }

  .transfer-section {
    margin-top: $padding-large-vertical;

    .row .pie-chart {
      margin-top: 10px;
    }
  }

  .magic-search-bar, .basic-search-bar {
    margin: $padding-small-vertical 0;
  }
}
