/*
 *    (c) Copyright 2015 Hewlett-Packard Development Company, L.P.
 *    (c) Copyright 2015 ThoughtWorks, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
(function () {
  'use strict';

  /**
   * @ngdoc overview
   * @name horizon.framework.util.workflow
   * @description
   *
   * # horizon.framework.util.workflow
   *
   * This module provides utility function service `workflow` to allow
   * decorating a workflow object.  A user (developer) friendly workflow
   * specification object can be shaped into a format that is friendly to
   * {@link horizon.framework.widgets.wizard `wizard`} by utilizing this service.
   *
   * The service provides a mechanism of decoupling general wizard UI component
   * from business components.
   *
   * | Factories                                                               |
   * |--------------------------------------------------------------------------|
   * | {@link horizon.framework.util.workflow.factory:workflow `workflow`}                |
   *
   */

  angular
    .module('horizon.framework.util.workflow', []);
})();
