{% load i18n %}

<div class="launchButtons pull-right">
  {% if launch_instance_allowed %}
    {% if show_ng_launch %}
      {% url 'horizon:project:network_topology:index' as networkUrl %}
      <a href="javascript:void(0);" ng-controller="LaunchInstanceModalController as modal"
         ng-click="modal.openLaunchInstanceWizard({successUrl: '{{networkUrl}}'})"
         id="instances__action_launch-ng" class="btn btn-default btn-launch
         {% if instance_quota_exceeded %}disabled{% endif %}">
        <span class="fa fa-cloud-upload"></span>
        {% if instance_quota_exceeded %}
          {% trans "Launch Instance (Quota exceeded)" %}
        {% else %}
          {% trans "Launch Instance" %}
        {% endif %}
      </a>
    {% endif %}
    {% if show_legacy_launch %}
      <a href="{% url 'horizon:project:network_topology:launchinstance' %}"
         id="instances__action_launch" class="btn btn-default btn-launch ajax-modal {% if instance_quota_exceeded %}disabled{% endif %}">
        <span class="fa fa-cloud-upload"></span>
        {% if instance_quota_exceeded %}
          {% trans "Launch Instance (Quota exceeded)" %}
        {% else %}
          {% trans "Launch Instance" %}
        {% endif %}
      </a>
    {% endif %}
  {% endif %}
  {% if create_network_allowed %}
    <a href="{% url 'horizon:project:network_topology:createnetwork' %}"
       id="networks__action_create" class="btn btn-default ajax-modal {% if network_quota_exceeded %}disabled{% endif %}">
      <span class="fa fa-plus"></span>
      {% if network_quota_exceeded %}
        {% trans "Create Network (Quota exceeded)" %}
      {% else %}
        {% trans "Create Network" %}
      {% endif %}
    </a>
  {% endif %}
  {% if create_router_allowed %}
    <a href="{% url 'horizon:project:network_topology:createrouter' %}"
       id="Routers__action_create" class="btn btn-default ajax-modal {% if router_quota_exceeded %}disabled{% endif %}">
      <span class="fa fa-plus"></span>
      {% if router_quota_exceeded %}
        {% trans "Create Router (Quota exceeded)" %}
      {% else %}
        {% trans "Create Router" %}
      {% endif %}
    </a>
  {% endif %}
</div>
