{% extends "base.html" %}
{% load i18n %}
{% block title %}{% trans "Disassociate Floating IP" %}{% endblock %}

{% block main %}
  {% include "project/instances/_disassociate.html" %}
{% endblock %}
