{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}

{% block modal-body-right %}
  <h3>{% trans "Description:" %}</h3>
  <p>{% blocktrans trimmed %}
    The status of a volume snapshot is normally managed automatically. In some circumstances
    an administrator may need to explicitly update the status value. This is equivalent to
    the <tt>openstack volume snapshot set --state</tt> command.
    {% endblocktrans %}</p>
{% endblock %}
