table[ng-controller="horizon.app.core.images.table.ImagesController as table"] {

  .detail-expanded .row {
    background: none;
    padding-left: 2em;
  }
}

.textarea-fixed-width {
  resize: vertical;
  min-height: 13em;
  max-height: 30em;
}

